package org.richfaces;

import org.ajax4jsf.test.base.RichSeleniumTest;
import org.ajax4jsf.test.base.SeleniumTestBase;
import org.testng.Assert;
import org.testng.annotations.AfterTest;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

public class ProgressBarTest extends SeleniumTestBase implements
		RichSeleniumTest {

	public ProgressBarTest() {
		super("http", "localhost", "8080");
	}

	/**
	 * This method are invoking before selenium tests started
	 */
	@BeforeTest
	@Parameters( { "browser" })
	public void startSelenium(String browser) {
		super.startSelenium(browser);
	}

	/**
	 * This method are invoking after selenium tests completed
	 */
	@AfterTest
	public void stopSelenium() {
		super.stopSelenium();
	}

	@Test
	public void testProgressBarComponent() {
		renderPage("/faces/pages/progressBar/progressBarTest.xhtml");

		_testProgressBarComponent(COMPONENT_PREFIX_INSIDE_TABLE);
		_testProgressBarComponent(COMPONENT_PREFIX_INSIDE_PANEL);
	}

	private void _testProgressBarComponent(String parentId) {
		
		//- Test ajax mode
		
		String progressBarId = parentId + "progressBar1";
		int value = getProgressBarValue(progressBarId);
		Assert.assertTrue(value < 0);
		String text = getTextById(progressBarId);
		Assert.assertTrue(text.startsWith("Process not started"));
		
		enableProgressBar(progressBarId, true);
		
		pause(5000, progressBarId);
		
		enableProgressBar(progressBarId, false);
		
		text = getTextById(progressBarId + ":remain");
		Assert.assertTrue(text.endsWith("%"));
		
		value = getProgressBarValue(progressBarId);
		Assert.assertTrue(value > 0);
		
		enableProgressBar(progressBarId, true);
		
		pause(5000, progressBarId);
		Assert.assertTrue((getProgressBarValue(progressBarId).intValue() != value));
		
		enableProgressBar(progressBarId, false);
		
		value = getProgressBarValue(progressBarId);
		delay(1500);
		Assert.assertTrue(getProgressBarValue(progressBarId) == value);
		
		
		// - Test client mode
		
		progressBarId = parentId + "progressBar2";
		value = getProgressBarValue(progressBarId);
		Assert.assertTrue(value == -5);
		
		text = getTextById(progressBarId);
		Assert.assertTrue(text.startsWith("Process not started"));
		
		setProgressBarValue(progressBarId, 20);
		value = getProgressBarValue(progressBarId);
		Assert.assertTrue(value == 20);
		
		setProgressBarValue(progressBarId, 60);
		value = getProgressBarValue(progressBarId);
		Assert.assertTrue(value == 60);
		
	}

	private void enableProgressBar(String id, boolean enable) {
		invokeFromComponent(id, (enable ? "enable" : "disable"), null);
	}

	private void setProgressBarValue(String id, Object value) {
		invokeFromComponent(id, "setValue", value);
	}

	private Integer getProgressBarValue(String id) {
		String value = invokeFromComponent(id, "getValue", null); //runScript(script.toString());
		Integer v = Integer.parseInt(value);
		return  v;
	}

}
